/*
 *  =======================================================================
 *  HumanOrgansGraph.java: Demo of JGraphT API. 
 * 
 *  Example code from Wicked Cool Java (No Starch Press)
 *  Copyright (C) 2005 Brian D. Eubanks
 * 
 *  Slightly modified by Mark Austin                          November 2011 
 *  =======================================================================
 */

package demo;

import java.util.List;
import java.util.Set;

import org.jgrapht.*;
import org.jgrapht.alg.*;
import org.jgrapht.graph.*;
import org.jgrapht.graph.DefaultDirectedGraph;

public class HumanOrgansGraph {

   enum  Organs { HEART, LUNG, LIVER, STOMACH, BRAIN, SPINALCORD, EYE, KIDNEY };
   enum Systems { CIRCULATORY, DIGESTIVE, NERVOUS, RESPIRATORY, IMMUNE };

   DefaultDirectedGraph graph = null;

   /*
    * Create an instance using the provided graph.
    * @param g The graph to use, or null to create a new one.
    */

   public HumanOrgansGraph( DefaultDirectedGraph g) {
      if (g == null) {
         g = new DefaultDirectedGraph<Enum, DefaultEdge> (DefaultEdge.class);
      }

      graph = g;

      // Add vertices to the graph

      g.addVertex(Organs.HEART);
      g.addVertex(Organs.LUNG);
      g.addVertex(Organs.BRAIN);
      g.addVertex(Organs.STOMACH);
      g.addVertex(Organs.LIVER);
      g.addVertex(Organs.SPINALCORD);

      g.addVertex(Systems.CIRCULATORY);
      g.addVertex(Systems.NERVOUS);
      g.addVertex(Systems.DIGESTIVE);
      g.addVertex(Systems.RESPIRATORY);

      // Link the vertices by edges

      g.addEdge(       Organs.HEART, Systems.CIRCULATORY);
      g.addEdge(        Organs.LUNG, Systems.RESPIRATORY);
      g.addEdge(       Organs.BRAIN,     Systems.NERVOUS);
      g.addEdge(  Organs.SPINALCORD,     Systems.NERVOUS);
      g.addEdge(     Organs.STOMACH,   Systems.DIGESTIVE);
      g.addEdge(       Organs.LIVER,   Systems.DIGESTIVE);

      // Simple test to see if certain vertices are in the graph ...

      System.out.println("");
      System.out.println("Test for existence of organs/systems");
      System.out.println("====================================");

      System.out.println("Test: Graph contains Systems.NERVOUS : " +
                         g.containsVertex ( Systems.NERVOUS ) );
      System.out.println("Test: Graph contains Organs.EYE : " +
                         g.containsVertex ( Organs.EYE ) );

      // Traverse the edges connected to DIGESTIVE vertex ...

      System.out.println("");
      System.out.println("Count set relationships in graph");
      System.out.println("====================================");

      Set digestiveLinks = g.edgesOf( Systems.DIGESTIVE );
      System.out.printf("There are %3d digestive organs in the graph\n", 
                        digestiveLinks.size() );
      Set lungLinks = g.edgesOf( Organs.LUNG );
      System.out.printf("The lung is part of %3d systems in the graph\n",
                        lungLinks.size() );

      // Walk along incoming edges and print source vertices ...

      System.out.println("");
      System.out.println("Print list of organs in the digestive system");
      System.out.println("============================================");

      for (Object item : digestiveLinks) {
           DefaultEdge edge = (DefaultEdge) item;
           Object source = graph.getEdgeSource( edge );
           System.out.println( "Source: " + source.toString() );
      }

      System.out.println("============================================");
   }

   // Exericse methods in human organ graph ...

   public static void main(String[] args) {

      System.out.println("Assemble Graph of Human Organ Systems");
      System.out.println("=====================================");

      new HumanOrgansGraph(null);
   }
}
