/*
 * =====================================================================
 * (C) Copyright 2012-2018, by Barak Naveh and Contributors.
 *
 * JGraphT : a free Java graph-theory library
 *
 * This program and the accompanying materials are dual-licensed under
 * either
 *
 * (a) the terms of the GNU Lesser General Public License version 2.1
 * as published by the Free Software Foundation, or (at your option) any
 * later version.
 *
 * or (per the licensee's choosing)
 *
 * (b) the terms of the Eclipse Public License v1.0 as published by
 * the Eclipse Foundation.
 * =====================================================================
 */

package demo;

import org.jgrapht.*;
import org.jgrapht.graph.*;

import java.util.*;

/**
 * An example of how to apply edge labels using a custom edge class.
 *
 * @author Barak Naveh
 * @since Jul 27, 2003
 */

public class LabeledEdges {
    private static final String FRIEND = "friend";
    private static final String ENEMY  = "enemy";

    /**
     * The starting point for the demo.
     *
     * @param args ignored.
     */

    public static void main(String[] args) {

        System.out.println("In demo.LabeledEdges.main() ...");
        System.out.println("==================================== ");

        // @example:create:begin

        Graph<String, RelationshipEdge> graph = new DefaultDirectedGraph<>(RelationshipEdge.class);

        ArrayList<String> people = new ArrayList<String>();
        people.add("John");
        people.add("James");
        people.add("Sarah");
        people.add("Jessica");

        // John is everyone's friend

        for (String person : people) {
            graph.addVertex(person);
            if (!person.equals("John")) {
                graph.addEdge( "John", person, new RelationshipEdge(FRIEND));
            }
        }

        // Apparently James doesn't really like John

        graph.addEdge("James", "John", new RelationshipEdge(ENEMY));

        // Jessica is Sarah and James's friend

        graph.addEdge("Jessica", "Sarah", new RelationshipEdge(FRIEND));
        graph.addEdge("Jessica", "James", new RelationshipEdge(FRIEND));

        // But Sarah really doesn't like James

        graph.addEdge("Sarah", "James", new RelationshipEdge(ENEMY));

        System.out.println("");
        System.out.println("Part 1: Relationship Graph");
        System.out.println("==========================");

        System.out.println (graph.toString() );

        // Print list of edge relationships

        System.out.println("");
        System.out.println("Part 2: Enumeration of Edge Relationships");
        System.out.println("=========================================");

        System.out.println ( graph.edgeSet().toString() );

        for (RelationshipEdge edge : graph.edgeSet()) {
            String v1 = graph.getEdgeSource(edge);
            String v2 = graph.getEdgeTarget(edge);
            if (edge.getLabel().equals("enemy")) {
                System.out.printf(v1 + " is an enemy of " + v2 + "\n");
            } else if (edge.getLabel().equals("friend")) {
                System.out.printf(v1 + " is a friend of " + v2 + "\n");
            }
        }

        assert( isEnemyOf(graph, "James", "John") );
    }

    //@example:isEnemyOf:begin

    private static boolean isEnemyOf( Graph<String, RelationshipEdge> graph,
                                      String person1, String person2) {
        return graph.getEdge(person1, person2).getLabel().equals(ENEMY);
    }
}
