/*
 * ============================================================================
 * TopologicalSorting03.java: Simulate assembly sequence for a stack of blocks.
 * ============================================================================
 */

package demo;

import java.util.Iterator;

import org.jgrapht.*;
import org.jgrapht.graph.*;
import org.jgrapht.traverse.TopologicalOrderIterator;
import org.jgrapht.graph.DefaultDirectedGraph;
import org.jgrapht.graph.DefaultEdge;
import org.jgrapht.traverse.*;

public class TopologicalSorting03 {

    public static void main ( String args[] ) {
        TopologicalSorting03 run = new TopologicalSorting03();
        run.blockExample();
    }

    public void blockExample() {

        DefaultDirectedGraph<String, DefaultEdge> graph = new DefaultDirectedGraph<String, DefaultEdge>( DefaultEdge.class );

        String v[] = new String[12];

        v[0] = "ColA1";
        v[2] = "ColA2";
        v[4] = "ColA3";

        v[1] = "COlB1";
        v[3] = "ColB2";
        v[5] = "ColB3";

        v[7] = "Foundation";
        v[6] = "Pier Cap";

        v[8]  = "Footing1";
        v[9]  = "Footing2";
        v[10] = "Footing3";
        v[11] = "Footing4";

        // Throw blocks into library in random order ...

        graph.addVertex(v[11]);
        graph.addVertex(v[10]);
        graph.addVertex(v[9]);
        graph.addVertex(v[8]);

        graph.addVertex(v[7]);
        graph.addVertex(v[6]);
        graph.addVertex(v[0]);
        graph.addVertex(v[1]);
        graph.addVertex(v[2]);
        graph.addVertex(v[3]);
        graph.addVertex(v[4]);
        graph.addVertex(v[5]);

        // =====================================
        // Specify dependencies among blocks ...
        // =====================================

        // Need lay foundation before building the columns ....

        graph.addEdge(v[7], v[0]);
        graph.addEdge(v[7], v[1]);

        // Specify assembly order for Pier A...

        graph.addEdge(v[0], v[2]);
        graph.addEdge(v[2], v[4]);

        // Specify assembly order for Pier B...

        graph.addEdge(v[1], v[3]);
        graph.addEdge(v[3], v[5]);

        // Add pier cap to columns ...

        graph.addEdge(v[4], v[6]);
        graph.addEdge(v[5], v[6]);

        // Put footings under foundation ...

        graph.addEdge( v[8], v[7]);
        graph.addEdge( v[9], v[7]);
        graph.addEdge(v[10], v[7]);
        graph.addEdge(v[11], v[7]);

        graph.addEdge( v[8],  v[9]);
        graph.addEdge( v[9], v[10]);

        Iterator<String> iter = new TopologicalOrderIterator<String, DefaultEdge>(graph);

        System.out.println("Sequence of Assembly");
        System.out.println("====================");

        int i = 0;
        while (iter.hasNext() != false) {
            String s = (String) iter.next();
            System.out.println( s.toString() );
        }
    }
}

// End TopologicalOrderIteratorTest.java
